/*
 * msglog.c
 *
 * SYSLOG -- print message on log file
 *
 * This routine looks a lot like printf, except that it outputs to the
 * log file instead of the standard output.  Also:
 *  adds a timestamp,
 *  prints the module name in front of the message,
 *  has some other formatting types (or will sometime),
 *  adds a newline on the end of the message.
 *
 * The output of this routine is intended to be read by syslogd(8).
 *
 *  This software is provided on an AS-IS basis.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/signal.h>
#include <sys/syslog.h>

#include <sys/uio.h>
#include <sys/wait.h>
#include <netdb.h>
#include <string.h>
#include <strings.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <fcntl.h>
#define _PATH_CONSOLE "/dev/console"

extern char logname[],deflogname[];
extern int  Debug;
static FILE *LogFile ;
static int  connected;      /* have done connect */
static int  LogStat = 0;        /* status bits, set by openlog() */
static const char *LogTag = "msglog";   /* string to tag the entry with */
static int  LogFacility = LOG_USER; /* default facility code */
void closelog (void);
void openlog (const char *ident, int logstat, int logfac);

void msglog(int pri, const char *fmt, ...)
{
   va_list ap;
   
   va_start(ap, fmt);
   vmsglog(pri, fmt, ap);
   va_end(ap);
}

vmsglog(int pri, const char *fmt, va_list ap)
{

   int cnt;
   char *p;
   time_t now;
   int fd, saved_errno;
   char tbuf[2048], fmt_cpy[1024], *stdp;
   
   if (pri > Debug) return;
   
   saved_errno = errno;
   
   if (LogFile < 0 || !connected)
       openlog(LogTag, LogStat | LOG_NDELAY, 0);
   
       /* build the message */
   
   (void)time(&now);
   (void)sprintf(tbuf, "<%d>%.15s ", pri, ctime(&now) + 4);
   for (p = tbuf; *p; ++p);
   if (LogTag)
       {
       (void)strcpy(p, LogTag);
       for (; *p; ++p);
       }
   if (LogStat & LOG_PID)
       {
       (void)sprintf(p, "[%d]", getpid());
       for (; *p; ++p);
    }
   if (LogTag)
       {
       *p++ = ':';
       *p++ = ' ';
    }

    /* substitute error message for %m */
   {
   register char ch, *t1, *t2;
   char *strerror();
   
   for (t1 = fmt_cpy; ch = *fmt; ++fmt)
       {
       if (ch == '%' && fmt[1] == 'm')
           {
           ++fmt;
           for (t2 = strerror(saved_errno);
               *t1 = *t2++; ++t1);
           }
       else
           *t1++ = ch;
       }
   *t1 = '\0';
   }
   
   (void)vsprintf(p, fmt_cpy, ap);
   
   cnt = strlen(tbuf);
   tbuf[cnt++] = '\n';
   tbuf[cnt] = '\0';
   
       /* output the message to the local logger */
       /* Use NUL as a message delimiter. */
   
   if (fwrite( tbuf, cnt + 1,1,LogFile) >= 1)
       {
       fflush(LogFile);
       return;
       }
   else        /* If the write fails, we try to reconect it next time. */
       closelog ();
   
       /* output the message to the console; don't worry about */
       /* blocking, if console blocks everything will. */
   
   if ((fd = open(_PATH_CONSOLE, O_WRONLY | O_NOCTTY, 0)) < 0)
       return;
   (void)strcat(tbuf, "\r\n");
   cnt += 2;
   p = index(tbuf, '>') + 1;
   (void)write( fd,p, cnt - (p - tbuf));
   (void)close(fd);
}

    /* OPENLOG -- open system log */

void openlog(const char *ident, int logstat, int logfac)
{
   if (logname[0] == '\0')
       strcpy(logname,deflogname);
   LogFile = fopen(logname,"a");
   connected = 1;
}

/* CLOSELOG -- close the message log */

void closelog (void)
{
   (void) fclose(LogFile);
   LogFile = NULL;
   connected = 0;
}

/* ======================= end of file ==================== */
